/**
 * 
 */
package com.windstream.bus.portal.security.jaas;

import java.lang.reflect.Method;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.login.LoginException;

import javax.servlet.http.HttpServletRequest;

import org.exoplatform.container.ExoContainer;

import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.log.Log;
import org.exoplatform.services.security.Authenticator;
import org.exoplatform.services.security.Credential;
import org.exoplatform.services.security.Identity;
import org.exoplatform.services.security.PasswordCredential;
import org.exoplatform.services.security.UsernameCredential;
import org.exoplatform.services.security.jaas.DefaultLoginModule;

import org.exoplatform.portal.application.PortalRequestContext;

import org.exoplatform.web.login.InitiateLoginServlet;

import org.exoplatform.web.security.GateInToken;
//import org.exoplatform.web.security.PortalLoginModule;
import org.exoplatform.web.security.security.TransientTokenService;
import org.exoplatform.web.security.security.CookieTokenService;

/**
 * The Class UAASLoginModule, customer LoginModule for Windstream.
 * 
 * @author jbragg
 */
public class UAASLoginModule extends DefaultLoginModule {

	private Authenticator authenticator;
	
    private static Method getContextMethod = null;
	
    /** The log. */
	protected Log log = ExoLogger.getLogger("UAASLoginModule");

	/**
	 * Instantiates a new uAAS login module.
	 */
	public UAASLoginModule() {
		super();
		System.out.println("Inside UAASLoginModuale constructor...");
		authenticator = new UAASAuthenticator();
	}

	public void addAuthenticatorPlugin(AuthenticatorPlugin authPlugin) {
		System.out.println("Adding new Authenticator to UAASLoginModuale...");
		log.info("Adding Authenticator for UAASLoginModule: "
				+ authPlugin.getAuthenticator().getClass().getName());
		authenticator = authPlugin.getAuthenticator();
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public boolean login() throws LoginException {
		if (log.isDebugEnabled())
			log.debug("In login of UAASLoginModule.");

		try {
			if (sharedState.containsKey("exo.security.identity")) {
				if (log.isDebugEnabled())
					log.debug("Use Identity from previous LoginModule");
				identity = (Identity) sharedState.get("exo.security.identity");
			} else {
				if (log.isDebugEnabled())
					log.debug("Try create identity");

				Callback[] callbacks = new Callback[2];
				callbacks[0] = new NameCallback("Username");
				callbacks[1] = new PasswordCallback("Password", true);

				callbackHandler.handle(callbacks);
				String username = ((NameCallback) callbacks[0]).getName();
				log.debug("***--- uaas: " + username);

				String password = new String(
						((PasswordCallback) callbacks[1]).getPassword());
				log.debug("***--- paas: " + password);

				((PasswordCallback) callbacks[1]).clearPassword();
				if (username == null || password == null)
					return false;

				// Authenticator authenticator =
				// (Authenticator)getContainer().getComponentInstanceOfType(Authenticator.class);
				if (authenticator == null)
					throw new LoginException(
							"No Authenticator component found, check your configuration");

				Credential[] credentials = null;

				org.exoplatform.web.security.Credentials creds = null;

				long tokenCnt = getTokenService().size();
				log.debug("***--# taas: " + tokenCnt);

				String[] allTokens = getTokenService().getAllTokens();

				if (allTokens != null) {

					for (String token : allTokens) {
						// if (log.isDebugEnabled()) {
						log.debug("Found Token: " + token);
						// }
					}
				} else {
					log.debug("Not Tokens Found.");
				}

				Object o = getTokenService().getToken(password);
				// check if token exist, if so get creds from token.
				if (o != null) {
					log.debug("***--- token: exists");
					GateInToken gToken = (GateInToken) o;
					creds = gToken.getPayload();
					credentials = new Credential[] {
							new UsernameCredential(creds.getUsername()),
							new PasswordCredential(creds.getPassword()) };
				} else {
					log.debug("***--- token: try validateToken");
					o = getTokenService().validateToken(password, false);
					if (o != null && (o instanceof org.exoplatform.web.security.Credentials)) {
						creds = (org.exoplatform.web.security.Credentials) o;
						credentials = new Credential[] {
								new UsernameCredential(creds.getUsername()),
								new PasswordCredential(creds.getPassword()) };
					} else {

						log.debug("***--- token: try Cookie");
						o = ((CookieTokenService) getContainer()
								.getComponentInstanceOfType(CookieTokenService.class))
								.validateToken(password, false);
						
						if (o != null && (o instanceof org.exoplatform.web.security.Credentials)) {
							creds = (org.exoplatform.web.security.Credentials) o;
							credentials = new Credential[] {
									new UsernameCredential(creds.getUsername()),
									new PasswordCredential(creds.getPassword()) };
						} else {
							log.debug("***--- token: try Session");
							if (getRequest() != null) {
								o = getRequest().getSession().getAttribute(
										InitiateLoginServlet.CREDENTIALS);
								if (o != null) {
									creds = (org.exoplatform.web.security.Credentials) o;
									credentials = new Credential[] {
											new UsernameCredential(
													creds.getUsername()),
											new PasswordCredential(
													creds.getPassword()) };
	
								} else {
									log.debug("***--- token: not found");
									// build credentials from username/password
									credentials = new Credential[] {
											new UsernameCredential(username),
											new PasswordCredential(password)
									};
								}
							} else {
								log.debug("***--- token: Request null");
								// build credentials from username/password
								credentials = new Credential[] {
										new UsernameCredential(username),
										new PasswordCredential(password) };
							}
						}
					}
				}

				String userId = authenticator.validateUser(credentials);
				identity = authenticator.createIdentity(userId);
				sharedState.put("javax.security.auth.login.name", userId);

				// TODO use PasswordCredential wrapper
				subject.getPrivateCredentials().add(password);
				subject.getPublicCredentials().add(
						new UsernameCredential(username));
			}
			return true;

		} catch (final Throwable e) {
			if (log.isDebugEnabled()) {
				log.debug("ERROR in UAASLoginModule: login(): "
						+ e.getMessage());
			}
			if (log.isWarnEnabled()) {
				log.warn("ERROR in UAASLoginModule: login(): " + e.getMessage());
			}
			if (log.isErrorEnabled()) {
				log.error("ERROR in UAASLoginModule: login(): "
						+ e.getMessage());
			}
			e.printStackTrace();
			throw new LoginException(e.getMessage());
		}
	}

	/**
	 * Gets the token service.
	 * 
	 * @return the token service
	 */
	private TransientTokenService getTokenService() {
		// TransientTokenService ttService = null;
		//
		// ExoContainer container = getContainer();
		// ttService = container
		// .getComponentInstanceOfType(TransientTokenService.class);
		// return TransientTokenService.getInstance(ttService.getClass());
		return TransientTokenService.getInstance(TransientTokenService.class);
	}

	private HttpServletRequest getRequest() throws Exception {
//		PortalRequestContext context = PortalRequestContext
//				.getCurrentInstance();
//		HttpServletRequest request = null;
//
//		if (context != null) {
//			request = (HttpServletRequest) context.getRequest();
//		}
//		return request;
		 return (HttpServletRequest)getContextMethod.invoke(null,
		 "javax.servlet.http.HttpServletRequest");
	}
}
